
from django import forms
from django.contrib import admin
from django.shortcuts import render, redirect
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages


def related_field_change_action(model, field_name, action_name):

    class FieldForm(forms.Form):

        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)

        value = forms.ModelChoiceField(
            label=model._meta.verbose_name,
            queryset=model.objects.all())

    def field_change_action(modeladmin, request, queryset):

        data = request.POST

        apply = 'apply' in data

        initial = {
            '_selected_action': data.getlist(admin.ACTION_CHECKBOX_NAME)
        }

        form = FieldForm(
            data=data if apply else None,
            initial=initial if not apply else None)

        if apply and form.is_valid():

            queryset.update(**{field_name: form.cleaned_data['value']})

            messages.success(request, _('Changes saved'))

            return redirect(request.get_full_path())

        context = admin.site.each_context(request)

        context.update({
            'items': queryset,
            'form': form,
            'action_name': 'field_change_action'
        })

        return render(
            request,
            'admin/change_related_field_action.html',
            context)

    field_change_action.short_description = action_name

    return field_change_action




from django.conf.global_settings import DATE_INPUT_FORMATS


def setup_settings(settings, **kwargs):

    settings['INSTALLED_APPS'] += [
        app for app in [
            'cap.configs.SuitConfig',
            'django.contrib.admin.apps.SimpleAdminConfig',
            'adminplus',
            'easy_select2'
        ] if app not in settings['INSTALLED_APPS']
    ]

    if 'DATE_INPUT_FORMATS' not in settings:
        settings['DATE_INPUT_FORMATS'] = DATE_INPUT_FORMATS

    date_format = '%d/%m/%Y'

    if date_format not in settings['DATE_INPUT_FORMATS']:
        settings['DATE_INPUT_FORMATS'].append(date_format)




from django.utils.translation import ugettext_lazy as _
from django.contrib.admin.filters import SimpleListFilter, FieldListFilter


class InputFilter(SimpleListFilter):

    template = 'admin/input_filter.html'

    def lookups(self, request, model_admin):
        # Dummy, required to show the filter.
        return ((),)

    def choices(self, changelist):
        # Grab only the "all" option.
        all_choice = next(super().choices(changelist))
        all_choice['query_parts'] = (
            (k, v)
            for k, v in changelist.get_filters_params().items()
            if k != self.parameter_name
        )
        yield all_choice


class IsNullFieldListFilter(FieldListFilter):

    notnull_label = _('Is present')
    isnull_label = _('Is Null')

    def __init__(
            self,
            field,
            request,
            params,
            model,
            model_admin,
            field_path):

        self.lookup_kwarg = '%s__isnull' % field_path
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)

        super().__init__(
            field,
            request,
            params,
            model,
            model_admin,
            field_path)

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def choices(self, cl):
        choices = (
            (None, _('All')),
            ('False', self.notnull_label),
            ('True', self.isnull_label),
        )
        for lookup, title in choices:
            yield {
                'selected': self.lookup_val == lookup,
                'query_string': cl.get_query_string({
                    self.lookup_kwarg: lookup,
                }),
                'display': title,
            }



from datetime import timedelta
try:
    from django.urls import reverse
except:
    from django.core.urlresolvers import reverse
from django.test import TestCase

class RoutesTestCase(TestCase):
    def test_login(self):
        response = self.client.get(reverse('admin:login'))
        self.assertContains(response, 'Log in')
